/*********************************************
 * OPL 12.4.2 Model
 * Author: mwatson, pcacioppi
 * Creation Date: Dec 28, 2012 at 10:52:23 AM
 * 
 * This model is meant to replicate the model 
 * on pages 48-52 of Supply Chain Network Design
 * It is re-engineered to make thsi 
 *********************************************/

/* core data of the model */
{string} Warehouses =...;  //this is I in the book and called a facility
{string} Customers =...;  // this is J in the book
int MaxWarehousesP =...;  // this is P in the book on page 49
float Demand[Customers] =...;  // this is d in the book
float Distance[Warehouses][Customers]=...;  // this is dist in the book

/* parameter constants */
float HighServiceDist =...; // this constant measures the threshold for when something is high service
float AvgServiceDist =...; // this constant restricts the average service distance
float MaximumDist =...; // this is the maximum distance we'll allow

/* metric variables that describe aggregate properties of a solution*/
dvar float AverageServiceDistVar; // captures the average service distance 						   
dvar float HighServiceVar;			// captures the proportion of demand met with high service
dvar float MaximumDistVar;		// metric variable used to implement the maximum distance constraint
		                       // it's presence seems extraneous in this model,
		                       // but it's very useful in the next model

/* detail variables that describe a solution */
dvar boolean OpenVar[Warehouses];  // this is X in the book
dvar boolean AssignVar[Warehouses][Customers];  //this is Y in the book
 
// this model is just maximizing the proportion of demand that is met with high service
maximize HighServiceVar; 
 
subject to{

ctMaxDist: MaximumDistVar <= MaximumDist;
ctAvgServDist: AverageServiceDistVar <= AvgServiceDist;
/* the above constraints just restrict the two metrics that aren't part of the objective */

/* apply the constraints that set the "metric" variables correctly */
ctAvgServiceDistance:  
	sum(w in Warehouses, c in Customers)
	  Distance[w][c]*Demand[c]*AssignVar[w][c] == 
	AverageServiceDistVar*sum(c in Customers) Demand[c];
   // because AverageServiceDistVar is an average, need to multiply by total demand

ctHighServiceDistance:
	sum(w in Warehouses, c in Customers)
   (Distance[w][c] > HighServiceDist ? 0 : 1) * Demand[c] * AssignVar[w][c] ==
	HighServiceVar * sum(c in Customers)  Demand[c] / 100.00; 
   // because HighServiceVar is a proportion, need to multiply by total demand divided by 100
   
forall (w in Warehouses, c in Customers)
		ctMaximumDistanceConstraint:
		AssignVar[w][c] * Distance[w][c] <= MaximumDistVar   ;

/* apply the logical constraints that make sure the model makes sense */
  forall ( c in Customers )
    ctEachCustomersDemandMustBeMet:
    	sum( w in Warehouses )
    	  AssignVar[w][c]==1;	
  
    ctOnlyUse_P_Warehouses:
  	sum(w in Warehouses)
  	  OpenVar[w]==MaxWarehousesP;
  
    forall (w in Warehouses, c in Customers)
    ctCannotAssignCustomertoWH_UnlessItIsOpen:
      AssignVar[w][c] <= OpenVar[w];
}